"""Mock data for demo mode and testing.

Provides realistic EOS device responses for a 6-device topology:
- spine-01, spine-02 (spines, AS 65000)
- leaf-01, leaf-02 (MLAG pair, AS 65001)
- leaf-03, leaf-04 (MLAG pair, AS 65002)
"""
